"""The LibreNMS module interacts with the LibreNMS instance when
- Creating a device.
- Validating the input of a device.
- Terminating a device.
"""
import json
import logging

import requests

from gso import settings

logger = logging.getLogger(__name__)


class CfgStruct:
    pass


def _get_cfg():
    """Internal function to retrieve all needed configuration."""
    oss = settings.load_oss_params()
    cfg = CfgStruct()
    # Hack for later ease: 1st setattr will fill in the inner's dict
    cfg._hack = ""
    # Update inner dict
    cfg.__dict__.update(oss.MONITORING)
    assert cfg.__dict__ is not None

    # Add parameters on-the-fly
    cfg.headers = {"X-Auth-Token": cfg.LIBRENMS.token}

    sep = "/"
    if cfg.LIBRENMS.endpoint.endswith("/"):
        sep = ""
    cfg.base_url = f"{cfg.LIBRENMS.endpoint}{sep}api/v0"
    cfg.url_devices = f"{cfg.base_url}/devices"
    cfg.url_switches = f"{cfg.base_url}/devicegroups/switches"
    cfg.device_groups = cfg.LIBRENMS.DEVICE_GROUPS
    cfg.environment = oss.GENERAL.environment
    if cfg.environment.startswith("lab"):
        cfg_dg_rtr_lab = cfg.device_groups.routers_lab
        cfg.url_routers = f"{cfg.base_url}/devicegroups/{cfg_dg_rtr_lab}"
    elif cfg.environment.startswith("prod"):
        cfg_dg_rtr_prod = cfg.device_groups.routers_prod
        cfg.url_routers = f"{cfg.base_url}/devicegroups/{cfg_dg_rtr_prod}"

    return cfg


def validate_device(fqdn: str):
    """Function that validates the existence of a device in LibreNMS.

    :param FQDN of the device to validate.
    """
    CFG = _get_cfg()

    # Validate existence
    nms_result = requests.get(
        CFG.url_devices, headers=CFG.headers)
    assert nms_result is not None

    device_id = [x.get("device_id") for x in filter(lambda x: x.get("hostname") == fqdn,
               nms_result.json().get("devices"))]

    if len(device_id) != 1 or device_id[0] is None:
        error_msg = f"Device with FQDN={fqdn} is not registered in LibreNMS"
        print(error_msg)
        raise AssertionError(error_msg)

    # Validate correctness
    device_id = device_id[0]
    url_device = f"{CFG.url_devices}/{device_id}"
    logger.debug(f"Connecting to URL: {url_device}"
                 f"with headers: {CFG.headers}")
    nms_result = requests.get(
        url_device, headers=CFG.headers)
    logger.debug(f"LibreNMS response={nms_result.content}")

    if nms_result.status_code != 200:
        print(nms_result.content)
        raise AssertionError(nms_result.content)

    # nms_dev_sysname = nms_result.json().get("sysName")
    nms_dev_hostname = nms_result.json().get("devices")[0].get("hostname")
    if fqdn != nms_dev_hostname:
        error_msg = f"Device with FQDN={fqdn} may not be correctly "\
                f"registered in LibreNMS (expected FQDN: {nms_dev_hostname})"
        print(error_msg)
        raise AssertionError(error_msg)


def register_device(fqdn: str):
    """Function that registers a new device in LibreNMS.

    :param FQDN of the device to register.
    """
    CFG = _get_cfg()
    logger.debug(f"Registering FQDN={fqdn} in LibreNMS")

    device_data = {
        "display": fqdn,
        "hostname": fqdn,
        "sysName": fqdn,
        # "override_icmp_disable": "true",
        # IMPORTANT: uncomment if testing with FQDNs that are not reachable
        # from LibreNMS (e.g. ContainerLab routers)
        # "force_add": "true"
    }
    if CFG.SNMP.version == "v2c":
        device_data.update({
            "community": CFG.SNMP.V2.community,
            })

    elif CFG.SNMP.version == "v3":
        for key in [
                "authlevel", "authname", "authpass", "authalgo",
                "cryptopass", "cryptoalgo"]:
            device_data.update({key: getattr(CFG.SNMP.V3, key)})

    logger.debug(f"Connecting to URL: {CFG.url_devices}"
                 f"with headers: {CFG.headers} and"
                 f"payload: {device_data}")
    nms_result = requests.post(
        CFG.url_devices, headers=CFG.headers,
        data=json.dumps(device_data))
    logger.debug(f"LibreNMS response={nms_result.content}")

    if nms_result.status_code != 200:
        print(nms_result.content)
        raise AssertionError(nms_result.content)


def deregister_device(fqdn: str):
    """Function that reregisters a device from LibreNMS.

    :param FQDN of the device to deregister.
    """
    CFG = _get_cfg()
    logger.debug(f"Deregistering FQDN={fqdn} from LibreNMS")

    nms_result = requests.get(
        CFG.url_devices, headers=CFG.headers)
    assert nms_result is not None
    device_id = [x.get("device_id") for x in filter(lambda x: x.get("hostname") == fqdn,
               nms_result.json().get("devices"))]
    if len(device_id) != 1:
        return
    device_id = device_id[0]

    # https://docs.librenms.org/API/Devices/#endpoint-categories
    device_data = {
        "field": "disabled",
        "data": "1",
    }
    url_device = f"{CFG.url_devices}/{device_id}"
    logger.debug(f"Connecting to URL: {url_device}"
                 f"with headers: {CFG.headers} and"
                 f"payload: {device_data}")
    nms_result = requests.patch(
        url_device, headers=CFG.headers,
        data=json.dumps(device_data))
    logger.debug(f"LibreNMS response={nms_result.content}")

    # Fail silently if device was not registered
    if nms_result.status_code != 200:
        print(nms_result.content)
